<?php declare(strict_types=1);
/*
 * This file is part of PHPUnit.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
class MethodCallbackByReference
{
    public function bar(&$a, &$b, $c): void
    {
        Legacy::bar($a, $b, $c);
    }

    public function callback(&$a, &$b, $c): void
    {
        $b = 1;
    }
}
