namespace PHPUnit\TestFixture\MockObject;

use PHPUnit\Framework\MockObject\Api;

class MockClassGenerated
{
    use Api;

    public static function getConfigurableMethods(): array
    {
        return static::$__phpunit_configurableMethods;
    }
}
